@@ -1,13 +1,29 @@
 ## realRatingMatrix
 
-## coercion
+## coercions
 setAs("matrix", "realRatingMatrix",
-  function(from) new("realRatingMatrix", 
+  function(from) new("realRatingMatrix",
     data = dropNA(from)))
 
 setAs("realRatingMatrix", "matrix",
   function(from) dropNA2matrix(from@data))
 
+setAs("realRatingMatrix", "dgCMatrix",
+  function(from) as(from@data, "dgCMatrix"))
+
+setAs("dgCMatrix", "realRatingMatrix",
+  function(from) new("realRatingMatrix", data = dropNA(from)))
+
+setAs("realRatingMatrix", "dgTMatrix",
+  function(from) as(from@data, "dgTMatrix"))
+
+setAs("dgTMatrix", "realRatingMatrix",
+  function(from) new("realRatingMatrix",
+    data = dropNA(as(from, "dgCMatrix"))))
+
+setAs("realRatingMatrix", "ngCMatrix",
+  function(from) as(from@data, "ngCMatrix"))
+
 ## from a data.frame with columns user, item, rating
 ## this perserves 0s
 setAs("data.frame", "realRatingMatrix", function(from) {
@@ -15,38 +31,37 @@ setAs("data.frame", "realRatingMatrix", function(from) {
   item	<- from[,2]
   if(ncol(from)>=3) rating <- as.numeric(from[,3])
   else rating <- rep(1, length(item))
-  
+
   i <- factor(user)
   j <- factor(item)
-  
-  dgT <- new("dgTMatrix", i = as.integer(i)-1L, j = as.integer(j)-1L, 
+
+  dgT <- new("dgTMatrix", i = as.integer(i)-1L, j = as.integer(j)-1L,
     x = rating,
     Dim = c(length(levels(i)), length(levels(j))),
     Dimnames = list(levels(i),levels(j)))
-  
-  new("realRatingMatrix", data = as(dgT, "dgCMatrix"))
+
+  as(dgT, "realRatingMatrix")
 })
 
-setAs("realRatingMatrix", "dgCMatrix",
-  function(from) from@data)
-
-setAs("realRatingMatrix", "dgTMatrix",
-  function(from) as(from@data, "dgTMatrix"))
-
-setAs("realRatingMatrix", "ngCMatrix",
-  function(from) as(from@data, "ngCMatrix"))
+setAs("realRatingMatrix", "data.frame", function(from) {
+    trip <- as(from, "dgTMatrix")
+    data.frame(user = rownames(from)[trip@i+1L],
+      item = colnames(from)[trip@j+1L],
+      rating = trip@x)[order(trip@i),]
+})
 
 setMethod("getList", signature(from = "realRatingMatrix"),
   function(from, decode = TRUE, ratings = TRUE,...) {
-    trip <- as(from@data, "dgTMatrix")
-    lst <- split(trip@j+1L, factor(trip@i, 
+    trip <- as(from, "dgTMatrix")
+
+    lst <- split(trip@j+1L, factor(trip@i,
       levels=0:(nrow(trip)-1L)), drop=FALSE)
-    rts <- split(trip@x, factor(trip@i, 
+    rts <- split(trip@x, factor(trip@i,
       levels=0:(nrow(trip)-1L)), drop=FALSE)
-    
+
     if(decode) lst <- lapply(lst, function(y) colnames(from)[y])
     else names(lst) <- NULL
-    
+
     if(ratings) {
       for(i in 1:length(rts)) {
         names(rts[[i]]) <- lst[[i]]
@@ -54,7 +69,7 @@ setMethod("getList", signature(from = "realRatingMatrix"),
     }else{
       rts <- lst
     }
-    
+
     names(rts) <- rownames(from)
     rts
   })
@@ -68,23 +83,17 @@ setMethod("binarize", signature(x = "realRatingMatrix"),
     x <- drop0(x)
     if(is.null(colnames(x))) colnames(x) <- 1:ncol(x)
     x <- as(t(x), "ngCMatrix")
-    
+
     new("binaryRatingMatrix", data = as(x, "itemMatrix"))
   })
 
-
-## ratings
-setMethod("getRatings", signature(x = "realRatingMatrix"),
-  function(x) x@data@x )
-
-
 setMethod("removeKnownRatings", signature(x = "realRatingMatrix"),
   function(x, known) {
     if(!is(known, "ratingMatrix")) stop("known needs to be a ratingMatrix!")
-    
+
     if(nrow(x) != nrow(known))
       stop("removeKnownRatings: Number of rows in x and known do not match!")
-    
+
     ## FIXME: make sparse
     xm <- as(x, "matrix")
     xm[as(as(known,"ngCMatrix"), "matrix")] <- NA
@@ -95,7 +104,7 @@ setMethod("removeKnownRatings", signature(x = "realRatingMatrix"),
 
 ## compute standard deviation
 .dgC2list <- function(x, row=TRUE) {
-  if(row) x <- t(x)   
+  if(row) x <- t(x)
   lapply(2:length(x@p), FUN = function(i) {
     if(x@p[i-1L]==x@p[i]) numeric(0)
     else x@x[(x@p[i-1L]+1L):x@p[i]]
@@ -123,28 +132,28 @@ setMethod("colSds", signature(x = "realRatingMatrix"),
 ## Note: negative given means all-but-given
 setMethod(".splitKnownUnknown", signature(data="realRatingMatrix"),
   function(data, given) {
-    
+
     ## given might of length one or length(data)
     if(length(given)==1) given <- rep(given, nrow(data))
     nitems <- rowCounts(data)
-    
+
     allBut <- given < 0
     if(any(allBut)) {
       given[allBut] <- nitems[allBut] + given[allBut]
     }
-    
-    if(any(given>nitems)) stop("Not enough ratings for user" , 
-      paste(which(given>nitems), collapse = ", ")) 
-    
-    
+
+    if(any(given>nitems)) stop("Not enough ratings for user" ,
+      paste(which(given>nitems), collapse = ", "))
+
+
     ## we create a logical mask via a triplet Matrix
     trip <- as(data, "dgTMatrix")
-    items <- lapply(0:(nrow(data)-1), 
+    items <- lapply(0:(nrow(data)-1),
       function(i) which(trip@i == i))
-    
-    take <-  unlist(lapply(1:length(items), 
+
+    take <-  unlist(lapply(1:length(items),
       function(i) sample(items[[i]],given[i])))
-    
+
     tripUnknown <- trip
     tripUnknown@x <- tripUnknown@x[-take]
     tripUnknown@i <- tripUnknown@i[-take]
@@ -153,16 +162,14 @@ setMethod(".splitKnownUnknown", signature(data="realRatingMatrix"),
     tripKnown@x <- tripKnown@x[take]
     tripKnown@i <- tripKnown@i[take]
     tripKnown@j <- tripKnown@j[take]
-    
-    known <- new("realRatingMatrix", 
-      data = as(tripKnown, "dgCMatrix"))
-    unknown <- new("realRatingMatrix", 
-      data = as(tripUnknown, "dgCMatrix"))
-    
+
+    known <- new("realRatingMatrix",
+      data = dropNA(as(tripKnown, "dgCMatrix")))
+    unknown <- new("realRatingMatrix",
+      data = dropNA(as(tripUnknown, "dgCMatrix")))
+
     list(
       known = known,
       unknown = unknown
     )
   })
-
-
